﻿1  --
   2  -- BaleLoader
   3  -- This is the specialization for automatic bale loaders
   4  --
   5  -- @author  Stefan Geiger
   6  -- @date  12/10/09
   7  --
   8  -- Copyright (C) GIANTS Software GmbH, Confidential, All Rights Reserved.
   9  
  10  source("dataS/scripts/vehicles/specializations/BaleLoaderStateEvent.lua");
  11  BaleLoader = {};
  12  
  13  function BaleLoader.prerequisitesPresent(specializations)
  14      return SpecializationUtil.hasSpecialization(Fillable, specializations);
  15  end;
  16  
  17  BaleLoader.GRAB_MOVE_UP = 1;
  18  BaleLoader.GRAB_MOVE_DOWN = 2;
  19  BaleLoader.GRAB_DROP_BALE = 3;
  20  
  21  BaleLoader.EMPTY_NONE = 1;
  22  BaleLoader.EMPTY_TO_WORK = 2;
  23  BaleLoader.EMPTY_ROTATE_PLATFORM = 3;
  24  BaleLoader.EMPTY_ROTATE1 = 4;
  25  BaleLoader.EMPTY_CLOSE_GRIPPERS = 5;
  26  BaleLoader.EMPTY_HIDE_PUSHER1 = 6;
  27  BaleLoader.EMPTY_HIDE_PUSHER2 = 7;
  28  BaleLoader.EMPTY_ROTATE2 = 8;
  29  BaleLoader.EMPTY_WAIT_TO_DROP = 9;
  30  BaleLoader.EMPTY_WAIT_TO_SINK = 10;
  31  BaleLoader.EMPTY_SINK = 11;
  32  BaleLoader.EMPTY_CANCEL = 12;
  33  BaleLoader.EMPTY_WAIT_TO_REDO = 13;
  34  
  35  BaleLoader.CHANGE_DROP_BALES = 1;
  36  BaleLoader.CHANGE_SINK = 2;
  37  BaleLoader.CHANGE_EMPTY_REDO = 3;
  38  BaleLoader.CHANGE_EMPTY_START = 4;
  39  BaleLoader.CHANGE_EMPTY_CANCEL = 5;
  40  BaleLoader.CHANGE_MOVE_TO_WORK = 6;
  41  BaleLoader.CHANGE_MOVE_TO_TRANSPORT = 7;
  42  BaleLoader.CHANGE_GRAB_BALE = 8;
  43  BaleLoader.CHANGE_GRAB_MOVE_UP = 9;
  44  BaleLoader.CHANGE_GRAB_DROP_BALE = 10;
  45  BaleLoader.CHANGE_GRAB_MOVE_DOWN = 11;
  46  BaleLoader.CHANGE_FRONT_PUSHER = 12;
  47  BaleLoader.CHANGE_ROTATE_PLATFORM = 13;
  48  BaleLoader.CHANGE_EMPTY_ROTATE_PLATFORM = 14;
  49  BaleLoader.CHANGE_EMPTY_ROTATE1 = 15;
  50  BaleLoader.CHANGE_EMPTY_CLOSE_GRIPPERS = 16;
  51  BaleLoader.CHANGE_EMPTY_HIDE_PUSHER1 = 17;
  52  BaleLoader.CHANGE_EMPTY_HIDE_PUSHER2 = 18;
  53  BaleLoader.CHANGE_EMPTY_ROTATE2 = 19;
  54  BaleLoader.CHANGE_EMPTY_WAIT_TO_DROP = 20;
  55  BaleLoader.CHANGE_EMPTY_STATE_NIL = 21;
  56  BaleLoader.CHANGE_EMPTY_WAIT_TO_REDO = 22;
  57  
  58  
  59  BaleLoader.CHANGE_BUTTON_EMPTY = 23;
  60  BaleLoader.CHANGE_BUTTON_EMPTY_ABORT = 24;
  61  BaleLoader.CHANGE_BUTTON_WORK_TRANSPORT = 25;
  62  
  63  
  64  
  65  -- emptying:                             user input state switch    emptyState                  state switch automatically performed
  66  -- 1. move grabber to work position           CHANGE_EMPTY_START -> (EMPTY_TO_WORK)          -> CHANGE_EMPTY_ROTATE_PLATFORM ->
  67  -- 2. rotate platform                                               (EMPTY_ROTATE_PLATFORM)  -> CHANGE_EMPTY_ROTATE1 ->
  68  -- 3. rotate main to half rotation and move the bales to the back   (EMPTY_ROTATE1)          -> CHANGE_EMPTY_CLOSE_GRIPPERS ->
  69  -- 4. close grippers                                                (EMPTY_CLOSE_GRIPPERS)   -> CHANGE_EMPTY_HIDE_PUSHER1 ->
  70  -- 5. hide pushers part 1 (rotation)                                (EMPTY_HIDE_PUSHER1)     -> CHANGE_EMPTY_HIDE_PUSHER2 ->
  71  -- 6. hide pushers part 2 (translation)                             (EMPTY_HIDE_PUSHER2)     -> CHANGE_EMPTY_ROTATE2 ->
  72  -- 7. rotate main to full rotation                                  (EMPTY_ROTATE2)          -> CHANGE_EMPTY_WAIT_TO_DROP ->
  73  -- 8. wait for user input                                           (EMPTY_WAIT_TO_DROP)
  74  
  75  -- 2 options:
  76  
  77  -- option 1:
  78  -- 9a. rotate main to no rotation              CHANGE_EMPTY_CANCEL     -> (EMPTY_CANCEL)     -> CHANGE_EMPTY_WAIT_TO_REDO ->
  79  -- 10a. wait for user input                                         (EMPTY_WAIT_TO_REDO)
  80  --                                             CHANGE_EMPTY_REDO -> (EMPTY_ROTATE2)
  81  
  82  -- option 2:
  83  -- 9b.                                         CHANGE_DROP_BALES -> (EMPTY_WAIT_TO_SINK)
  84  -- 10b. wait for user input
  85  -- 11b. rotate main to no rotation             CHANGE_SINK       -> (EMPTY_SINK)             -> CHANGE_EMPTY_STATE_NIL
  86  
  87  
  88  function BaleLoader:load(xmlFile)
  89  
  90      self.doStateChange = BaleLoader.doStateChange;
  91  
  92      self.balesToLoad = {}
  93  
  94      self.balesToMount = {};
  95  
  96      self.isInWorkPosition = false;
  97      self.grabberIsMoving = false;
  98      self.allowGrabbing = false;
  99  	self.synchronizeFillLevel = false;
 100  
 101      self.rotatePlatformDirection = 0;
 102      self.frontBalePusherDirection = 0;
 103  
 104      self.emptyState = BaleLoader.EMPTY_NONE;
 105  
 106      self.itemsToSave ={}
 107      self.fillLevel = 0;
 108      self.fillLevelMax = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.capacity"), 0);
 109  	self.capacity = self.fillLevelMax;
 110  
 111      self.baleGrabber = {};
 112      self.baleGrabber.grabNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.baleGrabber#grabNode"));
 113  
 114      self.startBalePlace = {};
 115      self.startBalePlace.bales = {};
 116      self.startBalePlace.node = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.balePlaces#startBalePlace"));
 117      if self.startBalePlace.node ~= nil then
 118          if getNumOfChildren(self.startBalePlace.node) < 2 then
 119              self.startBalePlace.node = nil;
 120          else
 121              self.startBalePlace.origRot = {};
 122              self.startBalePlace.origTrans = {};
 123              for i=1, 2 do
 124                  local node = getChildAt(self.startBalePlace.node, i-1);
 125                  local x,y,z = getRotation(node);
 126                  self.startBalePlace.origRot[i] = {x,y,z};
 127                  local x,y,z = getTranslation(node);
 128                  self.startBalePlace.origTrans[i] = {x,y,z};
 129              end;
 130          end;
 131      end;
 132      self.startBalePlace.count = 0;
 133  
 134      self.currentBalePlace = 1;
 135      self.balePlaces = {};
 136      local i=0;
 137      while true do
 138          local key = string.format("vehicle.balePlaces.balePlace(%d)", i);
 139          if not hasXMLProperty(xmlFile, key) then
 140              break;
 141          end;
 142          local node = Utils.indexToObject(self.components, getXMLString(xmlFile, key.."#node"));
 143          if node ~= nil then
 144              local entry = {};
 145              entry.node = node;
 146              table.insert(self.balePlaces, entry);
 147          end;
 148          i = i + 1;
 149      end;
 150  
 151      self.baleGrabSound = {};
 152      local baleGrabSoundFile = getXMLString(xmlFile, "vehicle.baleGrabSound#file");
 153      if baleGrabSoundFile ~= nil then
 154          baleGrabSoundFile = Utils.getFilename(baleGrabSoundFile, self.baseDirectory);
 155          self.baleGrabSound.sample = createSample("baleGrabSound");
 156          loadSample(self.baleGrabSound.sample,  baleGrabSoundFile, false);
 157          local pitch = getXMLFloat(xmlFile, "vehicle.baleGrabSound#pitchOffset");
 158          if pitch ~= nil then
 159              setSamplePitch(self.baleGrabSound.sample, pitch);
 160          end;
 161          self.baleGrabSound.volume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.baleGrabSound#volume"), 1);
 162      end;
 163  
 164      self.baleGrabParticleSystems = {};
 165      local psName = "vehicle.baleGrabParticleSystem";
 166      Utils.loadParticleSystem(xmlFile, self.baleGrabParticleSystems, psName, self.components, false, nil, self.baseDirectory);
 167      self.baleGrabParticleSystemDisableTime = 0;
 168      self.baleGrabParticleSystemDisableDuration = Utils.getNoNil(getXMLFloat(xmlFile, psName.."#disableDuration"), 0.6)*1000;
 169  
 170      self.baleLoaderHydraulicSound = {};
 171      local baleLoaderHydraulicSoundFile = getXMLString(xmlFile, "vehicle.baleLoaderHydraulicSound#file");
 172      if baleLoaderHydraulicSoundFile ~= nil then
 173          baleLoaderHydraulicSoundFile = Utils.getFilename(baleLoaderHydraulicSoundFile, self.baseDirectory);
 174          self.baleLoaderHydraulicSound.sample = createSample("baleLoaderHydraulicSound");
 175          loadSample(self.baleLoaderHydraulicSound.sample, baleLoaderHydraulicSoundFile, false);
 176          local pitch = getXMLFloat(xmlFile, "vehicle.baleLoaderHydraulicSound#pitchOffset");
 177          if pitch ~= nil then
 178              setSamplePitch(self.baleLoaderHydraulicSound.sample, pitch);
 179          end;
 180          self.baleLoaderHydraulicSound.volume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.baleLoaderHydraulicSound#volume"), 1);
 181          self.baleLoaderHydraulicSound.enabled = false;
 182      end;
 183  
 184      self.workTransportButton = InputBinding.IMPLEMENT_EXTRA;
 185      self.emptyAbortButton = InputBinding.IMPLEMENT_EXTRA2;
 186      self.emptyButton = InputBinding.IMPLEMENT_EXTRA3;
 187  
 188      self.baleTypes = {};
 189      local i = 0;
 190      while true do
 191          local key = string.format("vehicle.baleTypes.baleType(%d)", i);
 192          if not hasXMLProperty(xmlFile, key) then
 193              break;
 194          end;
 195          local filename = getXMLString(xmlFile, key.."#filename");
 196          if filename ~= nil then
 197              table.insert(self.baleTypes, filename);
 198          end;
 199          i = i + 1;
 200      end;
 201      if table.getn(self.baleTypes) == 0 then
 202          table.insert(self.baleTypes, "data/maps/models/objects/strawbale/strawbaleBaler.i3d");
 203      end;
 204  
 205  	self.baleloaderUVScrollParts = {};
 206      local i = 0;
 207      while true do
 208          local key = string.format("vehicle.baleloaderUVScrollParts.baleloaderUVScrollPart(%d)", i);
 209          if not hasXMLProperty(xmlFile, key) then
 210              break;
 211          end;
 212          local node = Utils.indexToObject(self.components, getXMLString(xmlFile, key.."#index"));
 213          local speed = Utils.getVectorNFromString(getXMLString(xmlFile, key.."#speed"), 2);
 214          if node ~= nil and speed then
 215              table.insert(self.baleloaderUVScrollParts, {node=node, speed=speed});
 216          end;
 217          i = i +1;
 218      end;
 219  end;
 220  
 221  function BaleLoader:delete()
 222  
 223      if (self.baleGrabParticleSystems) then
 224          Utils.deleteParticleSystem(self.baleGrabParticleSystems);
 225      end;
 226  
 227      -- avoid the bale nodes to be deleted twice (because they are linked the vehicle and because the Bale object is deleted)
 228      for i, balePlace in pairs(self.balePlaces) do
 229          if balePlace.bales ~= nil then
 230              for _, baleServerId in pairs(balePlace.bales) do
 231                  local bale = networkGetObject(baleServerId);
 232                  if bale ~= nil then
 233                      if bale.nodeId ~= 0 then
 234                          unlink(bale.nodeId);
 235                      end;
 236                  end;
 237              end;
 238          end;
 239      end;
 240      for i, baleServerId in ipairs(self.startBalePlace.bales) do
 241          local bale = networkGetObject(baleServerId);
 242          if bale ~= nil then
 243              if bale.nodeId ~= 0 then
 244                  unlink(bale.nodeId);
 245              end;
 246          end;
 247      end;
 248      if self.baleGrabSound.sample ~= nil then
 249          delete(self.baleGrabSound.sample);
 250          self.baleGrabSound.sample = nil;
 251      end;
 252      if self.baleLoaderHydraulicSound.sample ~= nil then
 253          delete(self.baleLoaderHydraulicSound.sample);
 254          self.baleLoaderHydraulicSound.sample = nil;
 255      end;
 256  end;
 257  
 258  function BaleLoader:readStream(streamId, connection)
 259  
 260      self.isInWorkPosition = streamReadBool(streamId);
 261      self.frontBalePusherDirection = streamReadIntN(streamId, 3);
 262      self.rotatePlatformDirection = streamReadIntN(streamId, 3);
 263  
 264      -- note: we do not sync grabberMoveState, this may lead to visual artifacts for a few seconds at the bigging
 265  
 266      if self.isInWorkPosition then
 267          BaleLoader.moveToWorkPosition(self);
 268      end;
 269  
 270      local emptyState = streamReadUIntN(streamId, 4);
 271  
 272      self.currentBalePlace = streamReadInt8(streamId);
 273      self.fillLevel = streamReadInt8(streamId);
 274  
 275      -- read bale at bale grabber
 276      if streamReadBool(streamId) then
 277          local baleServerId = streamReadInt32(streamId);
 278  
 279          self.baleGrabber.currentBale = baleServerId;
 280          self.balesToMount[baleServerId] = {serverId=baleServerId, linkNode=self.baleGrabber.grabNode, trans={0,0,0}, rot={0,0,0} };
 281      end;
 282  
 283      -- read bales at start bale places
 284      self.startBalePlace.count = streamReadUIntN(streamId, 3);
 285      for i=1, self.startBalePlace.count do
 286          local baleServerId = streamReadInt32(streamId);
 287  
 288          local attachNode = getChildAt(self.startBalePlace.node, i-1)
 289          self.balesToMount[baleServerId] = {serverId=baleServerId, linkNode=attachNode, trans={0,0,0}, rot={0,0,0} };
 290  
 291          table.insert(self.startBalePlace.bales, baleServerId);
 292      end;
 293  
 294  
 295      -- read bales at normal bale places
 296      for i=1, table.getn(self.balePlaces) do
 297          local balePlace = self.balePlaces[i];
 298  
 299          local numBales = streamReadUIntN(streamId, 3);
 300          if numBales > 0 then
 301              balePlace.bales = {};
 302  
 303              for baleI=1, numBales do
 304                  local baleServerId = streamReadInt32(streamId, baleServerId);
 305                  local x = streamReadFloat32(streamId);
 306                  local y = streamReadFloat32(streamId);
 307                  local z = streamReadFloat32(streamId);
 308  
 309                  table.insert(balePlace.bales, baleServerId);
 310  
 311                  self.balesToMount[baleServerId] = {serverId=baleServerId, linkNode=balePlace.node, trans={ x,y,z}, rot={0,0,0} };
 312              end;
 313          end;
 314      end;
 315  
 316      -- read num bales on platform
 317      -- read bales on baleloader
 318      -- ignore
 319  
 320      -- grabberMoveState ? int:nil
 321      -- isInWorkPosition  bool
 322      -- emptyState ? int:nil
 323      -- rotatePlatformDirection
 324      -- frontBalePusherDirection
 325      -- grabberIsMoving: bool
 326  
 327  
 328      -- update animations
 329      BaleLoader.updateBalePlacesAnimations(self);
 330  
 331      if emptyState >= BaleLoader.EMPTY_TO_WORK then
 332          self:doStateChange(BaleLoader.CHANGE_EMPTY_START);
 333          AnimatedVehicle.updateAnimations(self, 99999999);
 334          if emptyState >= BaleLoader.EMPTY_ROTATE_PLATFORM then
 335              self:doStateChange(BaleLoader.CHANGE_EMPTY_ROTATE_PLATFORM);
 336              AnimatedVehicle.updateAnimations(self, 99999999);
 337              if emptyState >= BaleLoader.EMPTY_ROTATE1 then
 338                  self:doStateChange(BaleLoader.CHANGE_EMPTY_ROTATE1);
 339                  AnimatedVehicle.updateAnimations(self, 99999999);
 340                  if emptyState >= BaleLoader.EMPTY_CLOSE_GRIPPERS then
 341                      self:doStateChange(BaleLoader.CHANGE_EMPTY_CLOSE_GRIPPERS);
 342                      AnimatedVehicle.updateAnimations(self, 99999999);
 343                      if emptyState >= BaleLoader.EMPTY_HIDE_PUSHER1 then
 344                          self:doStateChange(BaleLoader.CHANGE_EMPTY_HIDE_PUSHER1);
 345                          AnimatedVehicle.updateAnimations(self, 99999999);
 346                          if emptyState >= BaleLoader.EMPTY_HIDE_PUSHER2 then
 347                              self:doStateChange(BaleLoader.CHANGE_EMPTY_HIDE_PUSHER2);
 348                              AnimatedVehicle.updateAnimations(self, 99999999);
 349                              if emptyState >= BaleLoader.EMPTY_ROTATE2 then
 350                                  self:doStateChange(BaleLoader.CHANGE_EMPTY_ROTATE2);
 351                                  AnimatedVehicle.updateAnimations(self, 99999999);
 352                                  if emptyState >= BaleLoader.EMPTY_WAIT_TO_DROP then
 353                                      self:doStateChange(BaleLoader.CHANGE_EMPTY_WAIT_TO_DROP);
 354                                      AnimatedVehicle.updateAnimations(self, 99999999);
 355                                      if emptyState == BaleLoader.EMPTY_CANCEL or emptyState == BaleLoader.EMPTY_WAIT_TO_REDO then
 356                                          self:doStateChange(BaleLoader.CHANGE_EMPTY_CANCEL);
 357                                          AnimatedVehicle.updateAnimations(self, 99999999);
 358                                          if emptyState == BaleLoader.EMPTY_WAIT_TO_REDO then
 359                                              self:doStateChange(BaleLoader.CHANGE_EMPTY_WAIT_TO_REDO);
 360                                              AnimatedVehicle.updateAnimations(self, 99999999);
 361                                          end;
 362                                      elseif emptyState == BaleLoader.EMPTY_WAIT_TO_SINK or emptyState == BaleLoader.EMPTY_SINK then
 363                                          self:doStateChange(BaleLoader.CHANGE_DROP_BALES);
 364                                          AnimatedVehicle.updateAnimations(self, 99999999);
 365  
 366                                          if emptyState == BaleLoader.EMPTY_SINK then
 367                                              self:doStateChange(BaleLoader.CHANGE_SINK);
 368                                              AnimatedVehicle.updateAnimations(self, 99999999);
 369                                          end;
 370                                      end;
 371                                  end;
 372  
 373                              end;
 374                          end;
 375                      end;
 376                  end;
 377              end;
 378          end;
 379      end;
 380      self.emptyState = emptyState;
 381  end;
 382  
 383  function BaleLoader:writeStream(streamId, connection)
 384      streamWriteBool(streamId, self.isInWorkPosition);
 385      streamWriteIntN(streamId, self.frontBalePusherDirection, 3);
 386      streamWriteIntN(streamId, self.rotatePlatformDirection, 3);
 387  
 388      streamWriteUIntN(streamId, self.emptyState, 4);
 389  
 390      streamWriteInt8(streamId, self.currentBalePlace);
 391      streamWriteInt8(streamId, self.fillLevel);
 392  
 393      -- write bale at bale grabber
 394      if streamWriteBool(streamId, self.baleGrabber.currentBale ~= nil) then
 395          streamWriteInt32(streamId, self.baleGrabber.currentBale);
 396      end;
 397  
 398      -- write bales at start bale places
 399      streamWriteUIntN(streamId, self.startBalePlace.count, 3);
 400      for i=1, self.startBalePlace.count do
 401          local baleServerId = self.startBalePlace.bales[i];
 402          streamWriteInt32(streamId, baleServerId);
 403      end;
 404  
 405      -- write bales at normal bale places
 406      for i=1, table.getn(self.balePlaces) do
 407          local balePlace = self.balePlaces[i];
 408  
 409          local numBales = 0;
 410          if balePlace.bales ~= nil then
 411              numBales = table.getn(balePlace.bales);
 412          end;
 413          streamWriteUIntN(streamId, numBales, 3);
 414          if balePlace.bales ~= nil then
 415              for baleI=1, numBales do
 416                  local baleServerId = balePlace.bales[baleI];
 417                  local bale = networkGetObject(baleServerId);
 418                  local nodeId = bale.nodeId;
 419                  local x,y,z = getTranslation(nodeId);
 420                  streamWriteInt32(streamId, baleServerId);
 421                  streamWriteFloat32(streamId, x);
 422                  streamWriteFloat32(streamId, y);
 423                  streamWriteFloat32(streamId, z);
 424              end;
 425          end;
 426      end;
 427  end;
 428  
 429  function BaleLoader.updateBalePlacesAnimations(self)
 430      if self.currentBalePlace > 2 then
 431          -- currentBalePlace-1 is needs to be at the first position
 432          self:playAnimation("moveBalePlaces", 20, 0, true);
 433          self:setAnimationStopTime("moveBalePlaces", (self.currentBalePlace-2)/table.getn(self.balePlaces));
 434  
 435          AnimatedVehicle.updateAnimations(self, 99999999);
 436      end;
 437  
 438      if self.startBalePlace.count >= 1 then
 439  		self:playAnimation("balesToOtherRow", 20, nil, true);
 440          AnimatedVehicle.updateAnimations(self, 99999999);
 441          if self.startBalePlace.count >= 2 then
 442  			BaleLoader.rotatePlatform(self);
 443          end;
 444      end;
 445  end;
 446  
 447  function BaleLoader:mouseEvent(posX, posY, isDown, isUp, button)
 448  
 449  end;
 450  
 451  function BaleLoader:draw()
 452  
 453      if self:getIsActiveForInput(true) then
 454          if self.emptyState == BaleLoader.EMPTY_NONE then
 455              if self.grabberMoveState == nil then
 456                  if self.isInWorkPosition then
 457                      g_currentMission:addHelpButtonText(g_i18n:getText("BALELOADER_TRANSPORT"), self.workTransportButton);
 458                  else
 459                      g_currentMission:addHelpButtonText(g_i18n:getText("BALELOADER_WORK"), self.workTransportButton);
 460                  end;
 461              end;
 462  
 463              if BaleLoader.getAllowsStartUnloading(self) then
 464                  g_currentMission:addHelpButtonText(g_i18n:getText("BALELOADER_UNLOAD"), self.emptyButton);
 465              end;
 466          else
 467              if self.emptyState >= BaleLoader.EMPTY_TO_WORK and self.emptyState <= BaleLoader.EMPTY_ROTATE2 then
 468                  g_currentMission:addExtraPrintText(g_i18n:getText("BALELOADER_UP"));
 469              elseif self.emptyState == BaleLoader.EMPTY_CANCEL or self.emptyState == BaleLoader.EMPTY_SINK then
 470                  g_currentMission:addExtraPrintText(g_i18n:getText("BALELOADER_DOWN"));
 471              elseif self.emptyState == BaleLoader.EMPTY_WAIT_TO_DROP then
 472                  g_currentMission:addHelpButtonText(g_i18n:getText("BALELOADER_READY"), self.emptyButton);
 473                  g_currentMission:addHelpButtonText(g_i18n:getText("BALELOADER_ABORT"), self.emptyAbortButton);
 474              elseif self.emptyState == BaleLoader.EMPTY_WAIT_TO_SINK then
 475                  g_currentMission:addHelpButtonText(g_i18n:getText("BALELOADER_SINK"), self.emptyButton);
 476              elseif self.emptyState == BaleLoader.EMPTY_WAIT_TO_REDO then
 477                  g_currentMission:addHelpButtonText(g_i18n:getText("BALELOADER_UNLOAD"), self.emptyButton);
 478              end;
 479          end;
 480      end
 481  end
 482  
 483  
 484  function BaleLoader:keyEvent(unicode, sym, modifier, isDown)
 485  end;
 486  
 487  
 488  function BaleLoader:update(dt)
 489  
 490      if self.firstTimeRun then
 491          for k,v in pairs(self.balesToLoad) do
 492              local baleObject = Bale:new(self.isServer, self.isClient);
 493              local x,y,z = unpack(v.translation);
 494              local rx,ry,rz = unpack(v.rotation);
 495              baleObject:load(v.filename, x,y,z,rx,ry,rz, v.fillLevel);
 496              baleObject:mount(self, v.parentNode, x,y,z, rx,ry,rz)
 497              baleObject:register();
 498  
 499              table.insert(v.bales, networkGetObjectId(baleObject));
 500              self.balesToLoad[k] = nil;
 501          end;
 502  
 503          for k, baleToMount in pairs(self.balesToMount) do
 504              local bale = networkGetObject(baleToMount.serverId);
 505              if bale ~= nil then
 506                  local x,y,z = unpack(baleToMount.trans);
 507                  local rx,ry,rz = unpack(baleToMount.rot);
 508  
 509                  --print("mounting bale delayed: "..baleToMount.serverId..", pos: "..x.." "..y.." "..z);
 510                  bale:mount(self, baleToMount.linkNode, x,y,z, rx,ry,rz);
 511                  self.balesToMount[k] = nil;
 512              end;
 513          end;
 514      end;
 515  
 516      if self:getIsActive() then
 517          if self:getIsActiveForInput() and self.isClient then
 518              if InputBinding.hasEvent(self.emptyButton)  then
 519                  g_client:getServerConnection():sendEvent(BaleLoaderStateEvent:new(self, BaleLoader.CHANGE_BUTTON_EMPTY));
 520              elseif InputBinding.hasEvent(self.emptyAbortButton) then
 521                  g_client:getServerConnection():sendEvent(BaleLoaderStateEvent:new(self, BaleLoader.CHANGE_BUTTON_EMPTY_ABORT));
 522              elseif InputBinding.hasEvent(self.workTransportButton) then
 523                  g_client:getServerConnection():sendEvent(BaleLoaderStateEvent:new(self, BaleLoader.CHANGE_BUTTON_WORK_TRANSPORT));
 524              end;
 525          end;
 526  
 527          if self.isClient then
 528              if self.baleGrabParticleSystemDisableTime ~= 0 and self.baleGrabParticleSystemDisableTime < self.time then
 529                  Utils.setEmittingState(self.baleGrabParticleSystems, false);
 530                  self.baleGrabParticleSystemDisableTime = 0;
 531              end;
 532          end;
 533  
 534          -- check if grabber is still moving
 535          if self.grabberIsMoving then
 536              if not self:getIsAnimationPlaying("baleGrabberTransportToWork") then
 537                  self.grabberIsMoving = false;
 538              end;
 539          end;
 540  
 541          self.allowGrabbing = false;
 542          if self.isInWorkPosition and not self.grabberIsMoving and self.grabberMoveState == nil and self.startBalePlace.count < 2 and self.frontBalePusherDirection == 0 and self.rotatePlatformDirection == 0 and self.emptyState == BaleLoader.EMPTY_NONE and self.fillLevel < self.fillLevelMax then
 543              self.allowGrabbing = true;
 544          end;
 545  
 546          if self.isServer then
 547  
 548              if self.allowGrabbing then
 549                  if self.baleGrabber.grabNode ~= nil and self.baleGrabber.currentBale == nil then
 550                      -- find nearest bale
 551                      local nearestBale = BaleLoader.getBaleInRange(self, self.baleGrabber.grabNode);
 552                      if nearestBale ~= nil then
 553                          --BaleLoader.CHANGE_GRAB_BALE
 554                          g_server:broadcastEvent(BaleLoaderStateEvent:new(self, BaleLoader.CHANGE_GRAB_BALE, networkGetObjectId(nearestBale)), true, nil, self);
 555                      end;
 556                  end;
 557              end;
 558              if self.grabberMoveState ~= nil then
 559                  if self.grabberMoveState == BaleLoader.GRAB_MOVE_UP then
 560                      if not self:getIsAnimationPlaying("baleGrabberWorkToDrop") then
 561                          --BaleLoader.CHANGE_GRAB_MOVE_UP
 562                          -- switch to grabberMoveState
 563                          g_server:broadcastEvent(BaleLoaderStateEvent:new(self, BaleLoader.CHANGE_GRAB_MOVE_UP), true, nil, self);
 564                      end;
 565                  elseif self.grabberMoveState == BaleLoader.GRAB_DROP_BALE then
 566                      if not self:getIsAnimationPlaying("baleGrabberDropBale") then
 567                          --BaleLoader.CHANGE_GRAB_DROP_BALE
 568                          g_server:broadcastEvent(BaleLoaderStateEvent:new(self, BaleLoader.CHANGE_GRAB_DROP_BALE), true, nil, self);
 569                      end;
 570                  elseif self.grabberMoveState == BaleLoader.GRAB_MOVE_DOWN then
 571                      --BaleLoader.CHANGE_GRAB_MOVE_DOWN
 572                      if not self:getIsAnimationPlaying("baleGrabberWorkToDrop") then
 573                          g_server:broadcastEvent(BaleLoaderStateEvent:new(self, BaleLoader.CHANGE_GRAB_MOVE_DOWN), true, nil, self);
 574                      end;
 575                  end;
 576              end;
 577              if self.frontBalePusherDirection ~= 0 then
 578                  if not self:getIsAnimationPlaying("frontBalePusher") then
 579                      --BaleLoader.CHANGE_FRONT_PUSHER
 580                      g_server:broadcastEvent(BaleLoaderStateEvent:new(self, BaleLoader.CHANGE_FRONT_PUSHER), true, nil, self);
 581                  end;
 582              end;
 583              if self.rotatePlatformDirection ~= 0 then
 584                  if not self:getIsAnimationPlaying("rotatePlatform") then
 585                      --BaleLoader.CHANGE_ROTATE_PLATFORM
 586                      g_server:broadcastEvent(BaleLoaderStateEvent:new(self, BaleLoader.CHANGE_ROTATE_PLATFORM), true, nil, self);
 587                  end;
 588              end;
 589  
 590              if self.emptyState ~= BaleLoader.EMPTY_NONE then
 591                  if self.emptyState == BaleLoader.EMPTY_TO_WORK then
 592                      if not self:getIsAnimationPlaying("baleGrabberTransportToWork") then
 593                          g_server:broadcastEvent(BaleLoaderStateEvent:new(self, BaleLoader.CHANGE_EMPTY_ROTATE_PLATFORM), true, nil, self);
 594                      end;
 595                  elseif self.emptyState == BaleLoader.EMPTY_ROTATE_PLATFORM then
 596                      if not self:getIsAnimationPlaying("rotatePlatform") then
 597                          --BaleLoader.CHANGE_EMPTY_ROTATE1
 598                          g_server:broadcastEvent(BaleLoaderStateEvent:new(self, BaleLoader.CHANGE_EMPTY_ROTATE1), true, nil, self);
 599                      end;
 600                  elseif self.emptyState == BaleLoader.EMPTY_ROTATE1 then
 601                      if not self:getIsAnimationPlaying("emptyRotate") and not self:getIsAnimationPlaying("moveBalePlacesToEmpty") then
 602                          --BaleLoader.CHANGE_EMPTY_CLOSE_GRIPPERS
 603                          g_server:broadcastEvent(BaleLoaderStateEvent:new(self, BaleLoader.CHANGE_EMPTY_CLOSE_GRIPPERS), true, nil, self);
 604                      end;
 605                  elseif self.emptyState == BaleLoader.EMPTY_CLOSE_GRIPPERS then
 606                      if not self:getIsAnimationPlaying("closeGrippers") then
 607                          --BaleLoader.CHANGE_EMPTY_HIDE_PUSHER1
 608                          g_server:broadcastEvent(BaleLoaderStateEvent:new(self, BaleLoader.CHANGE_EMPTY_HIDE_PUSHER1), true, nil, self);
 609                      end;
 610                  elseif self.emptyState == BaleLoader.EMPTY_HIDE_PUSHER1 then
 611                      if not self:getIsAnimationPlaying("emptyHidePusher1") then
 612                          --BaleLoader.CHANGE_EMPTY_HIDE_PUSHER2
 613                          g_server:broadcastEvent(BaleLoaderStateEvent:new(self, BaleLoader.CHANGE_EMPTY_HIDE_PUSHER2), true, nil, self);
 614                      end;
 615                  elseif self.emptyState == BaleLoader.EMPTY_HIDE_PUSHER2 then
 616                      if self:getAnimationTime("moveBalePusherToEmpty") < 0.7 or not self:getIsAnimationPlaying("moveBalePusherToEmpty") then
 617                          --BaleLoader.CHANGE_EMPTY_ROTATE2
 618                          g_server:broadcastEvent(BaleLoaderStateEvent:new(self, BaleLoader.CHANGE_EMPTY_ROTATE2), true, nil, self);
 619                      end;
 620                  elseif self.emptyState == BaleLoader.EMPTY_ROTATE2 then
 621                      if not self:getIsAnimationPlaying("emptyRotate") then
 622                          --BaleLoader.CHANGE_EMPTY_WAIT_TO_DROP
 623                          g_server:broadcastEvent(BaleLoaderStateEvent:new(self, BaleLoader.CHANGE_EMPTY_WAIT_TO_DROP), true, nil, self);
 624                      end;
 625                  elseif self.emptyState == BaleLoader.EMPTY_SINK then
 626                      if not self:getIsAnimationPlaying("emptyRotate") and
 627                         not self:getIsAnimationPlaying("moveBalePlacesToEmpty") and
 628                         not self:getIsAnimationPlaying("emptyHidePusher1") and
 629                         not self:getIsAnimationPlaying("rotatePlatform")
 630                      then
 631                          --BaleLoader.CHANGE_EMPTY_STATE_NIL
 632                          g_server:broadcastEvent(BaleLoaderStateEvent:new(self, BaleLoader.CHANGE_EMPTY_STATE_NIL), true, nil, self);
 633                      end;
 634                  elseif self.emptyState == BaleLoader.EMPTY_CANCEL then
 635                      if not self:getIsAnimationPlaying("emptyRotate") then
 636                          --BaleLoader.CHANGE_EMPTY_WAIT_TO_REDO
 637                          g_server:broadcastEvent(BaleLoaderStateEvent:new(self, BaleLoader.CHANGE_EMPTY_WAIT_TO_REDO), true, nil, self);
 638                      end;
 639                  end;
 640              end;
 641          end;
 642  
 643          if self.isClient then
 644              if self.baleLoaderHydraulicSound.sample ~= nil then
 645                  local hasAnimationsPlaying = false;
 646                  for _, v in pairs(self.activeAnimations) do
 647                      hasAnimationsPlaying = true;
 648                      break;
 649                  end;
 650                  if hasAnimationsPlaying then
 651                      if not self.baleLoaderHydraulicSound.enabled and self:getIsActiveForSound() then
 652                          playSample(self.baleLoaderHydraulicSound.sample, 0, self.baleLoaderHydraulicSound.volume, 0);
 653                          self.baleLoaderHydraulicSound.enabled = true;
 654                      end;
 655                  else
 656                      if self.baleLoaderHydraulicSound.enabled then
 657                          stopSample(self.baleLoaderHydraulicSound.sample);
 658                          self.baleLoaderHydraulicSound.enabled = false;
 659                      end;
 660                  end;
 661              end;
 662          end;
 663      end;
 664  
 665  end;
 666  
 667  function BaleLoader.getBaleInRange(self, refNode)
 668      local px, py, pz = getWorldTranslation(refNode);
 669      local nearestDistance = 3.0;
 670      local nearestBale = nil;
 671  
 672      for index, item in pairs(g_currentMission.itemsToSave) do
 673          if item.item:isa(Bale) then
 674              -- TODO make with enum of bale types instead of filenames
 675              for _, filename in pairs(self.baleTypes) do
 676                  if item.item.i3dFilename == filename then
 677                      local vx, vy, vz = getWorldTranslation(item.item.nodeId);
 678                      local distance = Utils.vector3Length(px-vx, py-vy, pz-vz);
 679                      if distance < nearestDistance then
 680                          nearestDistance = distance;
 681                          nearestBale = item.item;
 682                      end;
 683                      break;
 684                  end;
 685              end;
 686          end;
 687      end;
 688      return nearestBale;
 689  end;
 690  
 691  function BaleLoader:onDetach()
 692      if self.deactivateOnDetach then
 693          BaleLoader.onDeactivate(self);
 694      else
 695          BaleLoader.onDeactivateSounds(self)
 696      end;
 697  end;
 698  
 699  function BaleLoader:onAttach()
 700  end;
 701  
 702  function BaleLoader:onDeactivate()
 703      Utils.setEmittingState(self.baleGrabParticleSystems, false);
 704  	for _, part in pairs(self.baleloaderUVScrollParts) do
 705          setShaderParameter(part.node, "uvScrollSpeed", 0, 0, 0, 0, false);
 706      end;
 707      BaleLoader.onDeactivateSounds(self)
 708  end;
 709  
 710  function BaleLoader:onDeactivateSounds()
 711      if self.baleLoaderHydraulicSound.sample ~= nil then
 712          stopSample(self.baleLoaderHydraulicSound.sample);
 713      end;
 714  end;
 715  
 716  
 717  function BaleLoader:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
 718      self.currentBalePlace = 1;
 719  
 720      self.startBalePlace.count = 0;
 721      local numBales = 0
 722      local i=0;
 723      while true do
 724          local baleKey = key..string.format(".bale(%d)", i);
 725          if not hasXMLProperty(xmlFile, baleKey) then
 726              break;
 727          end;
 728          local filename = getXMLString(xmlFile, baleKey.."#filename");
 729          if filename ~= nil then
 730              filename = Utils.convertFromNetworkFilename(filename);
 731  
 732              local x,y,z = Utils.getVectorFromString(getXMLString(xmlFile, baleKey.."#position"));
 733              local xRot,yRot,zRot = Utils.getVectorFromString(getXMLString(xmlFile, baleKey.."#rotation"));
 734              local fillLevel = getXMLFloat(xmlFile, baleKey.."#fillLevel");
 735              local balePlace = getXMLInt(xmlFile, baleKey.."#balePlace");
 736              local helper = getXMLInt(xmlFile, baleKey.."#helper");
 737              if balePlace == nil or (balePlace > 0 and (x == nil or y == nil or z == nil or xRot == nil or yRot == nil or zRot == nil)) or (balePlace < 1 and helper == nil) then
 738                  print("Warning: Corrupt savegame, bale "..filename.." could not be loaded");
 739              else
 740                  local translation;
 741                  local rotation;
 742                  if balePlace > 0 then
 743                      translation = {x,y,z};
 744                      rotation={xRot, yRot, zRot};
 745                  else
 746                      translation = {0,0,0};
 747                      rotation={0,0,0};
 748                  end;
 749                  local parentNode = nil;
 750                  local bales = nil;
 751                  if balePlace < 1 then
 752                      if helper <= getNumOfChildren(self.startBalePlace.node) then
 753                          parentNode = getChildAt(self.startBalePlace.node, helper-1);
 754                          if self.startBalePlace.bales == nil then
 755                              self.startBalePlace.bales = {};
 756                          end;
 757                          bales = self.startBalePlace.bales;
 758                          self.startBalePlace.count = self.startBalePlace.count+1;
 759                      end;
 760                  elseif balePlace <= table.getn(self.balePlaces) then
 761                      self.currentBalePlace = math.max(self.currentBalePlace, balePlace+1);
 762                      parentNode = self.balePlaces[balePlace].node;
 763                      if self.balePlaces[balePlace].bales == nil then
 764                          self.balePlaces[balePlace].bales = {};
 765                      end;
 766                      bales = self.balePlaces[balePlace].bales;
 767                  end;
 768                  if parentNode ~= nil then
 769                      numBales = numBales+1;
 770                      table.insert(self.balesToLoad, {parentNode=parentNode, filename=filename, bales=bales, translation=translation, rotation=rotation, fillLevel=fillLevel});
 771                  end;
 772              end;
 773          end;
 774          i = i +1;
 775      end;
 776  
 777      -- update animations
 778      BaleLoader.updateBalePlacesAnimations(self);
 779  
 780      self.fillLevel = numBales;
 781      return BaseMission.VEHICLE_LOAD_OK;
 782  end
 783  
 784  function BaleLoader:getSaveAttributesAndNodes(nodeIdent)
 785  
 786      local attributes = "";
 787      local nodes = "";
 788      local baleNum = 0;
 789  
 790      for i, balePlace in pairs(self.balePlaces) do
 791          if balePlace.bales ~= nil then
 792              for _, baleServerId in pairs(balePlace.bales) do
 793                  local bale = networkGetObject(baleServerId);
 794                  if bale ~= nil then
 795                      local nodeId = bale.nodeId;
 796                      local x,y,z = getTranslation(nodeId);
 797                      local rx,ry,rz = getRotation(nodeId);
 798                      if baleNum>0 then
 799                          nodes = nodes.."\n";
 800                      end;
 801                      nodes = nodes..nodeIdent..'<bale filename="'..Utils.encodeToHTML(Utils.convertToNetworkFilename(bale.i3dFilename))..'" fillLevel="'..bale:getFillLevel()..'" position="'..x..' '..y..' '..z..'" rotation="'..rx..' '..ry..' '..rz..'" balePlace="'..i..'" />';
 802                      baleNum = baleNum+1;
 803                  end;
 804              end;
 805          end;
 806      end;
 807      for i, baleServerId in ipairs(self.startBalePlace.bales) do
 808          local bale = networkGetObject(baleServerId);
 809          if bale ~= nil then
 810              if baleNum>0 then
 811                  nodes = nodes.."\n";
 812              end;
 813              nodes = nodes..nodeIdent..'<bale filename="'..Utils.encodeToHTML(Utils.convertToNetworkFilename(bale.i3dFilename))..'" fillLevel="'..bale:getFillLevel()..'" balePlace="0" helper="'..i..'"/>';
 814              baleNum = baleNum+1;
 815          end;
 816      end;
 817      return attributes,nodes;
 818  end
 819  
 820  function BaleLoader:doStateChange(id, nearestBaleServerId)
 821  
 822      if id == BaleLoader.CHANGE_DROP_BALES then
 823          -- drop all bales to ground (and add to save by mission)
 824          self.currentBalePlace = 1;
 825          for _, balePlace in pairs(self.balePlaces) do
 826              if balePlace.bales ~= nil then
 827                  for _, baleServerId in pairs(balePlace.bales) do
 828                      local bale = networkGetObject(baleServerId);
 829                      if bale ~= nil then
 830                          bale:unmount();
 831                      end;
 832                      self.balesToMount[baleServerId] = nil;
 833                  end;
 834                  balePlace.bales = nil;
 835              end;
 836          end;
 837          self.fillLevel = 0;
 838  		self:playAnimation("releaseFrontplattform", 1, nil, true);
 839          self:playAnimation("closeGrippers", -1, nil, true);
 840          self.emptyState = BaleLoader.EMPTY_WAIT_TO_SINK;
 841      elseif id == BaleLoader.CHANGE_SINK then
 842          self:playAnimation("emptyRotate", -1, nil, true);
 843          self:playAnimation("moveBalePlacesToEmpty", -5, nil, true);
 844          self:playAnimation("emptyHidePusher1", -1, nil, true);
 845          self:playAnimation("rotatePlatform", -1, nil, true);
 846          if not self.isInWorkPosition then
 847              self:playAnimation("closeGrippers", 1, self:getAnimationTime("closeGrippers"), true);
 848              self:playAnimation("baleGrabberTransportToWork", -1, nil, true);
 849          end;
 850          self.emptyState = BaleLoader.EMPTY_SINK;
 851      elseif id == BaleLoader.CHANGE_EMPTY_REDO then
 852          self:playAnimation("emptyRotate", 1, nil, true);
 853          self.emptyState = BaleLoader.EMPTY_ROTATE2;
 854      elseif id == BaleLoader.CHANGE_EMPTY_START then
 855          -- move to work position in case it is not there now
 856          BaleLoader.moveToWorkPosition(self);
 857          self.emptyState = BaleLoader.EMPTY_TO_WORK;
 858      elseif id == BaleLoader.CHANGE_EMPTY_CANCEL then
 859          self:playAnimation("emptyRotate", -1, nil, true);
 860          self.emptyState = BaleLoader.EMPTY_CANCEL;
 861      elseif id == BaleLoader.CHANGE_MOVE_TO_TRANSPORT then
 862          if self.isInWorkPosition then
 863              self.grabberIsMoving = true;
 864              self.isInWorkPosition = false;
 865              -- move to transport position
 866              BaleLoader.moveToTransportPosition(self);
 867  			for _, part in pairs(self.baleloaderUVScrollParts) do
 868  				setShaderParameter(part.node, "uvScrollSpeed", 0, 0, 0, 0, false);
 869  			end;
 870          end;
 871      elseif id == BaleLoader.CHANGE_MOVE_TO_WORK then
 872          if not self.isInWorkPosition then
 873              self.grabberIsMoving = true;
 874              self.isInWorkPosition = true;
 875              -- move to work position
 876              BaleLoader.moveToWorkPosition(self);
 877  			for _, part in pairs(self.baleloaderUVScrollParts) do
 878  				setShaderParameter(part.node, "uvScrollSpeed", part.speed[1], part.speed[2], 0, 0, false);
 879  			end;
 880          end;
 881      elseif id == BaleLoader.CHANGE_GRAB_BALE then
 882          local bale = networkGetObject(nearestBaleServerId);
 883          self.baleGrabber.currentBale = nearestBaleServerId;
 884          if bale ~= nil then
 885              bale:mount(self, self.baleGrabber.grabNode, 0,0,0, 0,0,0);
 886              self.balesToMount[nearestBaleServerId] = nil;
 887          else
 888              self.balesToMount[nearestBaleServerId] = {serverId=nearestBaleServerId, linkNode=self.baleGrabber.grabNode, trans={0,0,0}, rot={0,0,0} };
 889          end;
 890          self.grabberMoveState = BaleLoader.GRAB_MOVE_UP;
 891          self:playAnimation("baleGrabberWorkToDrop", 1, nil, true);
 892  
 893  		self.fillLevel = self.fillLevel + 1;
 894          if self.isClient then
 895              if self.baleGrabSound.sample ~= nil and self:getIsActiveForSound() then
 896                  playSample(self.baleGrabSound.sample, 1, self.baleGrabSound.volume, 0);
 897              end;
 898              Utils.setEmittingState(self.baleGrabParticleSystems, true);
 899              self.baleGrabParticleSystemDisableTime = self.time + self.baleGrabParticleSystemDisableDuration;
 900          end;
 901      elseif id == BaleLoader.CHANGE_GRAB_MOVE_UP then
 902          self:playAnimation("baleGrabberDropBale", 1, nil, true);
 903          self.grabberMoveState = BaleLoader.GRAB_DROP_BALE;
 904      elseif id == BaleLoader.CHANGE_GRAB_DROP_BALE then
 905          -- drop bale at platform
 906          if self.startBalePlace.count < 2 and self.startBalePlace.node ~= nil then
 907              local attachNode = getChildAt(self.startBalePlace.node, self.startBalePlace.count)
 908              local bale = networkGetObject(self.baleGrabber.currentBale);
 909              if bale ~= nil then
 910                  bale:mount(self, attachNode, 0,0,0, 0,0,0);
 911                  self.balesToMount[self.baleGrabber.currentBale] = nil;
 912              else
 913                  self.balesToMount[self.baleGrabber.currentBale] = {serverId=self.baleGrabber.currentBale, linkNode=attachNode, trans={0,0,0}, rot={0,0,0} };
 914              end;
 915  
 916              self.startBalePlace.count = self.startBalePlace.count + 1;
 917              table.insert(self.startBalePlace.bales, self.baleGrabber.currentBale);
 918              self.baleGrabber.currentBale = nil;
 919              --setRotation(baleNode, 0, 0, 0);
 920              --setTranslation(baleNode, 0, 0, 0);
 921  
 922              if self.startBalePlace.count == 1 then
 923                  self.frontBalePusherDirection = 1;
 924                  self:playAnimation("balesToOtherRow", 1, nil, true);
 925                  self:playAnimation("frontBalePusher", 1, nil, true);
 926              elseif self.startBalePlace.count == 2 then
 927                  BaleLoader.rotatePlatform(self);
 928              end;
 929  
 930              self:playAnimation("baleGrabberDropBale", -5, nil, true);
 931              self:playAnimation("baleGrabberWorkToDrop", -1, nil, true);
 932              self.grabberMoveState = BaleLoader.GRAB_MOVE_DOWN;
 933          end;
 934      elseif id == BaleLoader.CHANGE_GRAB_MOVE_DOWN then
 935          self.grabberMoveState = nil;
 936      elseif id == BaleLoader.CHANGE_FRONT_PUSHER then
 937          if self.frontBalePusherDirection > 0 then
 938              self:playAnimation("frontBalePusher", -1, nil, true);
 939              self.frontBalePusherDirection = -1;
 940          else
 941              self.frontBalePusherDirection = 0;
 942          end;
 943      elseif id == BaleLoader.CHANGE_ROTATE_PLATFORM then
 944          if self.rotatePlatformDirection > 0 then
 945              -- drop bales
 946              local balePlace = self.balePlaces[self.currentBalePlace];
 947              self.currentBalePlace = self.currentBalePlace + 1;
 948              for i=1, table.getn(self.startBalePlace.bales) do
 949                  local node = getChildAt(self.startBalePlace.node, i-1);
 950                  local x,y,z = getTranslation(node);
 951                  local baleServerId = self.startBalePlace.bales[i];
 952                  local bale = networkGetObject(baleServerId);
 953                  if bale ~= nil then
 954                      bale:mount(self, balePlace.node, x,y,z, 0,0,0);
 955                      self.balesToMount[baleServerId] = nil;
 956                  else
 957                      self.balesToMount[baleServerId] = {serverId=baleServerId, linkNode=balePlace.node, trans={ x,y,z}, rot={0,0,0} };
 958                  end;
 959              end;
 960              balePlace.bales = self.startBalePlace.bales;
 961              self.startBalePlace.bales = {};
 962              self.startBalePlace.count = 0;
 963  
 964              for i=1, 2 do
 965                  local node = getChildAt(self.startBalePlace.node, i-1);
 966                  setRotation(node, unpack(self.startBalePlace.origRot[i]));
 967                  setTranslation(node, unpack(self.startBalePlace.origTrans[i]));
 968              end;
 969  
 970              if self.emptyState == BaleLoader.EMPTY_NONE then
 971                  -- we are not waiting to start emptying, rotate back
 972                  self.rotatePlatformDirection = -1;
 973                  self:playAnimation("rotatePlatform", -1, nil, true);
 974              else
 975                  self.rotatePlatformDirection = 0;
 976              end;
 977          else
 978              self.rotatePlatformDirection = 0;
 979          end;
 980      elseif id == BaleLoader.CHANGE_EMPTY_ROTATE_PLATFORM then
 981          self.emptyState = BaleLoader.EMPTY_ROTATE_PLATFORM;
 982          if self.startBalePlace.count == 0 then
 983              self:playAnimation("rotatePlatform", 1, nil, true);
 984          else
 985              BaleLoader.rotatePlatform(self)
 986          end;
 987      elseif id == BaleLoader.CHANGE_EMPTY_ROTATE1 then
 988          self:playAnimation("emptyRotate", 1, nil, true);
 989          self:setAnimationStopTime("emptyRotate", 0.2);
 990          local balePlacesTime = self:getRealAnimationTime("moveBalePlaces");
 991          self:playAnimation("moveBalePlacesToEmpty", 1.5, balePlacesTime/self:getAnimationDuration("moveBalePlacesToEmpty"), true);
 992          self:playAnimation("moveBalePusherToEmpty", 1.5, balePlacesTime/self:getAnimationDuration("moveBalePusherToEmpty"), true);
 993          self.emptyState = BaleLoader.EMPTY_ROTATE1;
 994      elseif id == BaleLoader.CHANGE_EMPTY_CLOSE_GRIPPERS then
 995          self:playAnimation("closeGrippers", 1, nil, true);
 996          self.emptyState = BaleLoader.EMPTY_CLOSE_GRIPPERS;
 997      elseif id == BaleLoader.CHANGE_EMPTY_HIDE_PUSHER1 then
 998          self:playAnimation("emptyHidePusher1", 1, nil, true);
 999          self.emptyState = BaleLoader.EMPTY_HIDE_PUSHER1;
1000      elseif id == BaleLoader.CHANGE_EMPTY_HIDE_PUSHER2 then
1001          self:playAnimation("moveBalePusherToEmpty", -2, nil, true);
1002          self.emptyState = BaleLoader.EMPTY_HIDE_PUSHER2;
1003      elseif id == BaleLoader.CHANGE_EMPTY_ROTATE2 then
1004          self:playAnimation("emptyRotate", 1, self:getAnimationTime("emptyRotate"), true);
1005          self.emptyState = BaleLoader.EMPTY_ROTATE2;
1006      elseif id == BaleLoader.CHANGE_EMPTY_WAIT_TO_DROP then
1007          -- wait for the user to react (abort or drop)
1008          self.emptyState = BaleLoader.EMPTY_WAIT_TO_DROP;
1009      elseif id == BaleLoader.CHANGE_EMPTY_STATE_NIL then
1010          self.emptyState = BaleLoader.EMPTY_NONE;
1011      elseif id == BaleLoader.CHANGE_EMPTY_WAIT_TO_REDO then
1012          self.emptyState = BaleLoader.EMPTY_WAIT_TO_REDO;
1013      elseif id == BaleLoader.CHANGE_BUTTON_EMPTY then
1014          -- Server only code
1015          assert(self.isServer);
1016          if self.emptyState ~= BaleLoader.EMPTY_NONE then
1017              if self.emptyState == BaleLoader.EMPTY_WAIT_TO_DROP then
1018                  -- BaleLoader.CHANGE_DROP_BALES
1019                  g_server:broadcastEvent(BaleLoaderStateEvent:new(self, BaleLoader.CHANGE_DROP_BALES), true, nil, self);
1020              elseif self.emptyState == BaleLoader.EMPTY_WAIT_TO_SINK then
1021                  -- BaleLoader.CHANGE_SINK
1022                  g_server:broadcastEvent(BaleLoaderStateEvent:new(self, BaleLoader.CHANGE_SINK), true, nil, self);
1023              elseif self.emptyState == BaleLoader.EMPTY_WAIT_TO_REDO then
1024                  -- BaleLoader.CHANGE_EMPTY_REDO
1025                  g_server:broadcastEvent(BaleLoaderStateEvent:new(self, BaleLoader.CHANGE_EMPTY_REDO), true, nil, self);
1026              end;
1027          else
1028              --BaleLoader.CHANGE_EMPTY_START
1029              if BaleLoader.getAllowsStartUnloading(self) then
1030                  g_server:broadcastEvent(BaleLoaderStateEvent:new(self, BaleLoader.CHANGE_EMPTY_START), true, nil, self);
1031              end;
1032          end;
1033      elseif id == BaleLoader.CHANGE_BUTTON_EMPTY_ABORT then
1034          -- Server only code
1035          assert(self.isServer);
1036          if self.emptyState ~= BaleLoader.EMPTY_NONE then
1037              if self.emptyState == BaleLoader.EMPTY_WAIT_TO_DROP then
1038                  --BaleLoader.CHANGE_EMPTY_CANCEL
1039                  g_server:broadcastEvent(BaleLoaderStateEvent:new(self, BaleLoader.CHANGE_EMPTY_CANCEL), true, nil, self);
1040              end;
1041          end;
1042      elseif id == BaleLoader.CHANGE_BUTTON_WORK_TRANSPORT then
1043          -- Server only code
1044          assert(self.isServer);
1045          if self.emptyState == BaleLoader.EMPTY_NONE and self.grabberMoveState == nil then
1046              if self.isInWorkPosition then
1047                  g_server:broadcastEvent(BaleLoaderStateEvent:new(self, BaleLoader.CHANGE_MOVE_TO_TRANSPORT), true, nil, self);
1048              else
1049                  g_server:broadcastEvent(BaleLoaderStateEvent:new(self, BaleLoader.CHANGE_MOVE_TO_WORK), true, nil, self);
1050              end;
1051          end;
1052      end;
1053  end;
1054  
1055  function BaleLoader.getAllowsStartUnloading(self)
1056      return self.fillLevel > 0 and self.rotatePlatformDirection == 0 and self.frontBalePusherDirection == 0 and not self.grabberIsMoving and self.grabberMoveState == nil and self.emptyState == BaleLoader.EMPTY_NONE;
1057  end;
1058  
1059  function BaleLoader.rotatePlatform(self)
1060      self.rotatePlatformDirection = 1;
1061      self:playAnimation("rotatePlatform", 1, nil, true);
1062      --[[if self.startBalePlace.count > 0 then
1063          self:playAnimation("rotatePlatformMoveBales"..self.startBalePlace.count, 1,  nil, true);
1064      end;]]
1065      if self.currentBalePlace > 1 then
1066          -- currentBalePlace is needs to be at the first position
1067          self:playAnimation("moveBalePlaces", 1, (self.currentBalePlace-1)/table.getn(self.balePlaces), true);
1068          self:setAnimationStopTime("moveBalePlaces", (self.currentBalePlace)/table.getn(self.balePlaces));
1069      end;
1070  end;
1071  
1072  function BaleLoader.moveToWorkPosition(self)
1073      self:playAnimation("baleGrabberTransportToWork", 1, Utils.clamp(self:getAnimationTime("baleGrabberTransportToWork"), 0, 1), true);
1074      self:playAnimation("closeGrippers", -1, Utils.clamp(self:getAnimationTime("closeGrippers"), 0, 1), true);
1075  end;
1076  
1077  function BaleLoader.moveToTransportPosition(self)
1078      self:playAnimation("baleGrabberTransportToWork", -1, Utils.clamp(self:getAnimationTime("baleGrabberTransportToWork"), 0, 1), true);
1079      self:playAnimation("closeGrippers", 1, Utils.clamp(self:getAnimationTime("closeGrippers"), 0, 1), true);
1080  end;